@@ -1,6 +1,7 @@
 #ifndef SEQUENCE_H
 #define SEQUENCE_H 1
 
+#include <cstring>
 #include <stdint.h>
 #include <string>
 #include <cassert>
@@ -22,6 +23,15 @@ static inline bool isACGT(char c)
 		|| c == 'a' || c == 'c' || c == 'g' || c == 't';
 }
 
+/**
+ * Return true if a sequence consists entirely of ACGT chars
+ * (case insensitive).
+ */
+static inline bool allACGT(const Sequence& seq)
+{
+	return strspn(seq.c_str(), "acgtACGT") == seq.length();
+}
+
 /**
  * Convert each ambiguity code to the lexicographically smallest
  * matching base.
